@@ -36,5 +36,9 @@ ChangedGallery::~ChangedGallery()
 
 Glib::ustring ChangedGallery::get_change_description() const
 {
-  return ChangedDocument::get_change_description_for_items(first, second);
+  Glib::ustring extra;
+  if (first.get_private_audience() != second.get_private_audience())
+    extra = "audience";
+  return ChangedDocument::get_change_description_for_items(first, second, 
+                                                           extra);
 }
