@@ -25,15 +25,36 @@
 #include "document-dialog.h"
 #include "gallery-dialog.h"
 
-ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
+ChangesEditorDialog * ChangesEditorDialog::create(Tapioca::Session *s)
 {
-  session = s;
   Glib::RefPtr<Gtk::Builder> xml
     = Gtk::Builder::create_from_file(Main::get_glade_path()
 				    + "/changes-editor-dialog.gtk");
 
+  ChangesEditorDialog *dialog;
   xml->get_widget("dialog", dialog);
-  dialog->set_icon_from_file(Main::get_data_path() + "/icon.png");
+  dialog->on_dialog_loaded(s);
+  return dialog;
+}
+
+void ChangesEditorDialog::on_dialog_loaded(Tapioca::Session *s)
+{
+  session = s;
+  changes = session->get_changes();
+  fill_modified_galleries(changes.modified_galleries);
+  fill_modified_documents(changes.modified_docs);
+  fill_removed_galleries(changes.removed_galleries);
+  fill_added_galleries(changes.added_galleries);
+  fill_removed_documents(changes.removed_documents);
+  fill_added_documents(changes.added_documents);
+  update_buttons();
+}
+
+ChangesEditorDialog::ChangesEditorDialog(BaseObjectType* baseObject, 
+                                         const Glib::RefPtr<Gtk::Builder>& xml)
+ : Gtk::Dialog(baseObject)
+{
+  set_icon_from_file(Main::get_data_path() + "/icon.png");
   xml->get_widget("modified_galleries_expander", modified_galleries_expander);
   xml->get_widget("modified_galleries_treeview", modified_galleries_treeview);
   xml->get_widget("modified_galleries_label", modified_galleries_label);
@@ -91,7 +112,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_modified_gallery)));
-  modified_galleries_context_menu.accelerate(*dialog);
+  modified_galleries_context_menu.accelerate(*this);
 
   modified_documents_list = Gtk::ListStore::create(modified_documents_columns);
   modified_documents_treeview->set_model(modified_documents_list);
@@ -103,7 +124,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_modified_document)));
-  modified_documents_context_menu.accelerate(*dialog);
+  modified_documents_context_menu.accelerate(*this);
 
   removed_galleries_list = Gtk::ListStore::create(removed_galleries_columns);
   removed_galleries_treeview->set_model(removed_galleries_list);
@@ -118,7 +139,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_removed_gallery)));
-  removed_galleries_context_menu.accelerate(*dialog);
+  removed_galleries_context_menu.accelerate(*this);
 
   added_galleries_list = Gtk::ListStore::create(added_galleries_columns);
   added_galleries_treeview->set_model(added_galleries_list);
@@ -130,7 +151,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_added_gallery)));
-  added_galleries_context_menu.accelerate(*dialog);
+  added_galleries_context_menu.accelerate(*this);
 
   removed_documents_list = Gtk::ListStore::create(removed_documents_columns);
   removed_documents_treeview->set_model(removed_documents_list);
@@ -142,7 +163,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_removed_document)));
-  removed_documents_context_menu.accelerate(*dialog);
+  removed_documents_context_menu.accelerate(*this);
 
   added_documents_list = Gtk::ListStore::create(added_documents_columns);
   added_documents_treeview->set_model(added_documents_list);
@@ -154,16 +175,7 @@ ChangesEditorDialog::ChangesEditorDialog(Tapioca::Session *s)
       (Gtk::Menu_Helpers::MenuElem
        ("_Revert", 
         sigc::mem_fun(*this, &ChangesEditorDialog::on_revert_added_document)));
-  added_documents_context_menu.accelerate(*dialog);
-
-  changes = session->get_changes();
-  fill_modified_galleries(changes.modified_galleries);
-  fill_modified_documents(changes.modified_docs);
-  fill_removed_galleries(changes.removed_galleries);
-  fill_added_galleries(changes.added_galleries);
-  fill_removed_documents(changes.removed_documents);
-  fill_added_documents(changes.added_documents);
-  update_buttons();
+  added_documents_context_menu.accelerate(*this);
 }
 
 void ChangesEditorDialog::update_buttons()
@@ -309,34 +321,17 @@ void ChangesEditorDialog::add_row_added_document(Tapioca::ChangedDocument *docum
 
 ChangesEditorDialog::~ChangesEditorDialog()
 {
-  delete dialog;
-}
-
-void ChangesEditorDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void ChangesEditorDialog::hide()
-{
-  dialog->hide();
-}
-
-int ChangesEditorDialog::run()
-{
-  int response = dialog->run();
-  return response;
 }
 
 void ChangesEditorDialog::on_modified_gallery_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   Gallery *gallery = new Gallery(get_selected_modified_gallery()->first);
-  GalleryDialog d(gallery, &get_selected_modified_gallery()->second);
-  d.set_parent_window(*dialog);
-  int response = d.run();
-  d.hide();
+  GalleryDialog *d = GalleryDialog::create
+    (gallery, &get_selected_modified_gallery()->second);
+  d->set_transient_for(*this);
+  int response = d->run();
+  d->hide();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       Gallery *orig_gallery = &get_selected_modified_gallery()->first;
@@ -348,16 +343,18 @@ void ChangesEditorDialog::on_modified_gallery_activated(const Gtk::TreeModel::Pa
           update_selected_modified_gallery_changes();
         }
     }
+  delete d;
 }
 
 void ChangesEditorDialog::on_modified_document_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   Document *document = new Document(get_selected_modified_document()->first);
-  DocumentDialog d(document, &get_selected_modified_document()->second);
-  d.set_parent_window(*dialog);
-  int response = d.run();
-  d.hide();
+  DocumentDialog *d = DocumentDialog::create
+    (document, &get_selected_modified_document()->second);
+  d->set_transient_for(*this);
+  int response = d->run();
+  d->hide();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       Document *orig_document = &get_selected_modified_document()->first;
@@ -369,24 +366,29 @@ void ChangesEditorDialog::on_modified_document_activated(const Gtk::TreeModel::P
           update_selected_modified_document_changes();
         }
     }
+  delete d;
 }
 
 void ChangesEditorDialog::on_removed_gallery_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   ChangedGallery * gallery= get_selected_removed_gallery();
-  GalleryDialog d(&gallery->first, &gallery->second, true);
-  d.run();
+  GalleryDialog *d = GalleryDialog::create (&gallery->first, &gallery->second, 
+                                            true);
+  d->set_transient_for(*this);
+  d->run();
+  delete d;
 }
 
 void ChangesEditorDialog::on_added_gallery_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   Gallery *gallery = new Gallery(get_selected_added_gallery()->first);
-  GalleryDialog d(gallery, &get_selected_added_gallery()->second);
-  d.set_parent_window(*dialog);
-  int response = d.run();
-  d.hide();
+  GalleryDialog* d = GalleryDialog::create
+    (gallery, &get_selected_added_gallery()->second);
+  d->set_transient_for(*this);
+  int response = d->run();
+  d->hide();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       Gallery *orig_gallery = &get_selected_added_gallery()->first;
@@ -398,24 +400,29 @@ void ChangesEditorDialog::on_added_gallery_activated(const Gtk::TreeModel::Path&
           update_selected_added_gallery_name();
         }
     }
+  delete d;
 }
 
 void ChangesEditorDialog::on_removed_document_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   ChangedDocument* document = get_selected_removed_document();
-  DocumentDialog d(&document->first, &document->second, true);
-  d.run();
+  DocumentDialog *d = 
+    DocumentDialog::create(&document->first, &document->second, true);
+  d->set_transient_for(*this);
+  d->run();
+  delete d;
 }
 
 void ChangesEditorDialog::on_added_document_activated(const Gtk::TreeModel::Path& path,
                                                   Gtk::TreeViewColumn *col)
 {
   Document *document = new Document(get_selected_added_document()->first);
-  DocumentDialog d(document, &get_selected_added_document()->second);
-  d.set_parent_window(*dialog);
-  int response = d.run();
-  d.hide();
+  DocumentDialog *d = DocumentDialog::create
+    (document, &get_selected_added_document()->second);
+  d->set_transient_for(*this);
+  int response = d->run();
+  d->hide();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       Document *orig_document = &get_selected_added_document()->first;
@@ -427,6 +434,7 @@ void ChangesEditorDialog::on_added_document_activated(const Gtk::TreeModel::Path
           update_selected_added_document_name();
         }
     }
+  delete d;
 }
 
 void ChangesEditorDialog::update_selected_added_document_name()
@@ -608,6 +616,25 @@ void ChangesEditorDialog::on_revert_modified_document()
     }
 }
 
+void ChangesEditorDialog::remove_removed_parent_gallery(Tapioca::ChangedDocument *document)
+{
+  if (session->find_by_id(document->first.get_gallery_id()) != NULL)
+    return;
+
+  for (std::list<ChangedGallery>::iterator i = 
+       changes.removed_galleries.begin(); i != changes.removed_galleries.end();
+       i++)
+    {
+      if ((*i).first.get_id() == document->first.get_gallery_id())
+        {
+          reverts.removed_galleries.push_back(*i);
+          changes.removed_galleries.remove(*i);
+          fill_removed_galleries(changes.removed_galleries);
+          return;
+        }
+    }
+}
+
 void ChangesEditorDialog::on_revert_removed_document()
 {
   Glib::RefPtr<Gtk::TreeSelection> selection = removed_documents_treeview->get_selection();
@@ -616,6 +643,8 @@ void ChangesEditorDialog::on_revert_removed_document()
     {
       Gtk::TreeModel::Row row = *iterrow;
       ChangedDocument *document = row[removed_documents_columns.data];
+      remove_same_added_document(document);
+      remove_removed_parent_gallery(document);
       reverts.removed_documents.push_back(*document);
       changes.removed_documents.remove(*document);
       removed_documents_list->erase(iterrow);
@@ -681,27 +710,33 @@ void ChangesEditorDialog::remove_same_removed_document(Tapioca::ChangedDocument
   //this event shows up as two separate add, and remove changes.
   //so we need to remove the corresponding event, when we're dealing with a 
   //moved document.
-  std::list<ChangedDocument>::iterator it = 
-    std::find(changes.removed_documents.begin(), 
-              changes.removed_documents.end(), *document);
-  if (it != changes.removed_documents.end())
+  std::list<ChangedDocument>::iterator it = changes.removed_documents.begin();
+  for (; it != changes.removed_documents.end(); it++)
     {
-      changes.removed_documents.erase(it);
-      fill_removed_documents(changes.removed_documents);
-      update_buttons();
+      if ((*it).first.get_id() == document->first.get_id())
+        {
+          reverts.removed_documents.push_back(*it);
+          changes.removed_documents.erase(it);
+          fill_removed_documents(changes.removed_documents);
+          update_buttons();
+          break;
+        }
     }
 }
 
-void ChangesEditorDialog::remove_same_added_docment(Tapioca::ChangedDocument *document)
+void ChangesEditorDialog::remove_same_added_document(Tapioca::ChangedDocument *document)
 {
-  std::list<ChangedDocument>::iterator it = 
-    std::find(changes.added_documents.begin(), 
-              changes.added_documents.end(), *document);
-  if (it != changes.added_documents.end())
+  std::list<ChangedDocument>::iterator it = changes.added_documents.begin();
+  for (; it != changes.added_documents.end(); it++)
     {
-      changes.added_documents.erase(it);
-      fill_added_documents(changes.added_documents);
-      update_buttons();
+      if ((*it).first.get_id() == document->first.get_id())
+        {
+          reverts.added_documents.push_back(*it);
+          changes.added_documents.erase(it);
+          fill_added_documents(changes.added_documents);
+          update_buttons();
+          break;
+        }
     }
 }
 
@@ -716,7 +751,8 @@ void ChangesEditorDialog::remove_documents_belonging_to_reverted_removed_gallery
        changes.removed_documents.begin(); 
        i != changes.removed_documents.end(); i++)
     {
-      if ((*i).first.get_gallery_id() == gallery->get_id())
+      if ((*i).first.get_gallery_id() == gallery->get_id() &&
+          session->find_doc_by_id((*i).first.get_id()) == NULL) //was it moved?
         docs_to_remove.push_back(*i);
     }
 
@@ -748,6 +784,7 @@ void ChangesEditorDialog::remove_documents_belonging_to_reverted_added_gallery(T
   for (std::list<ChangedDocument>::iterator i = docs_to_remove.begin();
        i != docs_to_remove.end(); i++)
     {
+      remove_same_removed_document(&*i);
       reverts.added_documents.push_back(*i);
       changes.added_documents.remove(*i);
     }
@@ -757,3 +794,18 @@ void ChangesEditorDialog::remove_documents_belonging_to_reverted_added_gallery(T
       update_buttons();
     }
 }
+/*
+test cases:
+reverting a modified gallery
+reverting a modified document
+reverting an added gallery
+reverting an added document
+reverting an added gallery with a new document in it.
+reverting a removed document
+reverting a removed gallery with an existing document in it.
+reverting a moved document (when reverting removed document).
+reverting a moved document (when reverting added document).
+reverting a new gallery when it has a moved document in it.
+reverting a removed gallery when it has a document moved out of it.
+reverting a removed document that had a parent gallery also removed
+*/
