@@ -23,29 +23,45 @@
 #include "ucompose.hpp"
 #include "profile.h"
 
-ConnectDialog::ConnectDialog(Tapioca::Profile *p, bool modify_username)
+ConnectDialog* ConnectDialog::create(Tapioca::Profile *p, bool modify_username)
 {
-  profile = p;
   Glib::RefPtr<Gtk::Builder> xml
     = Gtk::Builder::create_from_file(Main::get_glade_path()
                                      + "/connect-dialog.gtk");
+  ConnectDialog *dialog;
+  xml->get_widget_derived("dialog", dialog);
+  dialog->on_dialog_loaded(p, modify_username);
+  return dialog;
+}
 
-  xml->get_widget("dialog", dialog);
-  dialog->set_icon_from_file(Main::get_data_path() + "/icon.png");
+ConnectDialog::ConnectDialog(BaseObjectType* baseObject, 
+                             const Glib::RefPtr<Gtk::Builder>& xml)
+ : Gtk::Dialog(baseObject)
+{
+
+  set_icon_from_file(Main::get_data_path() + "/icon.png");
 
   xml->get_widget("username_entry", username_entry);
   xml->get_widget("password_entry", password_entry);
   xml->get_widget("accept_button", accept_button);
 
   username_entry->signal_changed().connect
-    (sigc::mem_fun(this,&ConnectDialog::on_username_changed));
+    (sigc::mem_fun(this, &ConnectDialog::on_username_changed));
   password_entry->signal_changed().connect
-    (sigc::mem_fun(this,&ConnectDialog::on_password_changed));
+    (sigc::mem_fun(this, &ConnectDialog::on_password_changed));
+  signal_response().connect
+    (sigc::mem_fun(this, &ConnectDialog::on_response_received));
+}
 
-    
+void ConnectDialog::on_dialog_loaded(Tapioca::Profile *p, bool modify_username)
+{
+  profile = p;
+  username_locked = !modify_username;
   Glib::RefPtr<Gnome::Conf::Client> conf_client;
   conf_client = Gnome::Conf::Client::get_default_client();
-  username_entry->set_text(conf_client->get_string("/apps/" PACKAGE_NAME "/general/previous_username"));
+  username_entry->set_text
+    (conf_client->get_string
+     ("/apps/" PACKAGE_NAME "/general/previous_username"));
   if (username_entry->get_text() == "")
     username_entry->set_text(profile->get_username());
   password_entry->set_text(profile->get_password());
@@ -54,28 +70,16 @@ ConnectDialog::ConnectDialog(Tapioca::Profile *p, bool modify_username)
   else
     username_entry->grab_focus();
   update_buttons();
-  if (modify_username == false && username_entry->get_text() != "")
+  if (username_locked == true && username_entry->get_text() != "")
     username_entry->set_sensitive(false);
 }
 
 ConnectDialog::~ConnectDialog()
 {
-  delete dialog;
 }
 
-void ConnectDialog::set_parent_window(Gtk::Window &parent)
+void ConnectDialog::on_response_received(int response)
 {
-  dialog->set_transient_for(parent);
-}
-
-void ConnectDialog::hide()
-{
-  dialog->hide();
-}
-
-int ConnectDialog::run()
-{
-  int response = dialog->run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
       Glib::RefPtr<Gnome::Conf::Client> conf_client;
@@ -83,7 +87,6 @@ int ConnectDialog::run()
       conf_client->set("/apps/" PACKAGE_NAME "/general/previous_username", 
                        username_entry->get_text());
     }
-  return response;
 }
 
 void ConnectDialog::on_username_changed()
