@@ -22,31 +22,24 @@
 #include "main.h"
 #include "ucompose.hpp"
 
-DocumentDialog::DocumentDialog(Tapioca::Document *d,
-                               const Tapioca::Document *orig, bool uneditable)
+DocumentDialog* DocumentDialog::create(Tapioca::Document *d, const Tapioca::Document *orig, bool uneditable)
+{
+  Glib::RefPtr<Gtk::Builder> xml
+    = Gtk::Builder::create_from_file(Main::get_glade_path()
+                                     + "/document-dialog.gtk");
+  DocumentDialog *dialog;
+  xml->get_widget_derived("dialog", dialog);
+  dialog->on_dialog_loaded(d, orig, uneditable);
+  return dialog;
+}
+
+void DocumentDialog::on_dialog_loaded(Tapioca::Document *d,
+                                      const Tapioca::Document *orig, 
+                                      bool uneditable)
 {
   document = d;
   readonly = uneditable;
   original = orig;
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(Main::get_glade_path()
-                                     + "/document-dialog.gtk");
-
-  xml->get_widget("dialog", dialog);
-  dialog->set_icon_from_file(Main::get_data_path() + "/icon.png");
-
-  xml->get_widget("title_entry", title_entry);
-  xml->get_widget("description_textview", description_textview);
-  xml->get_widget("thumbnail_image", thumbnail_image);
-  xml->get_widget("lat_spinbutton", lat_spinbutton);
-  xml->get_widget("lon_spinbutton", lon_spinbutton);
-  xml->get_widget("item_tagging_widget_container", 
-                  item_tagging_widget_container);
-  xml->get_widget("title_label", title_label);
-  xml->get_widget("description_label", description_label);
-  xml->get_widget("latitude_label", latitude_label);
-  xml->get_widget("longitude_label", longitude_label);
-
   if (document)
     {
       title_entry->set_text(d->get_title());
@@ -56,15 +49,6 @@ DocumentDialog::DocumentDialog(Tapioca::Document *d,
       thumbnail_image->set(d->get_thumbnail());
     }
 
-  title_entry->signal_changed().connect
-    (sigc::mem_fun(this,&DocumentDialog::on_title_changed));
-  description_textview->get_buffer()->signal_changed().connect
-    (sigc::mem_fun(this, &DocumentDialog::on_description_changed));
-  lat_spinbutton->signal_value_changed().connect
-    (sigc::mem_fun(this,&DocumentDialog::on_lat_changed));
-  lon_spinbutton->signal_value_changed().connect
-    (sigc::mem_fun(this,&DocumentDialog::on_lon_changed));
-
   item_tagging_widget = new ItemTaggingWidget();
   item_tagging_widget->get_widget()->reparent(*item_tagging_widget_container);
   item_tagging_widget->fill_tags(document->get_tags());
@@ -81,35 +65,54 @@ DocumentDialog::DocumentDialog(Tapioca::Document *d,
   on_description_changed();
   on_lat_changed();
   on_lon_changed();
+  title_entry->signal_changed().connect
+    (sigc::mem_fun(this,&DocumentDialog::on_title_changed));
+  description_textview->get_buffer()->signal_changed().connect
+    (sigc::mem_fun(this, &DocumentDialog::on_description_changed));
+  lat_spinbutton->signal_value_changed().connect
+    (sigc::mem_fun(this,&DocumentDialog::on_lat_changed));
+  lon_spinbutton->signal_value_changed().connect
+    (sigc::mem_fun(this,&DocumentDialog::on_lon_changed));
+}
+
+DocumentDialog::DocumentDialog(BaseObjectType* baseObject, 
+                               const Glib::RefPtr<Gtk::Builder>& xml)
+ : Gtk::Dialog(baseObject)
+{
+
+  set_icon_from_file(Main::get_data_path() + "/icon.png");
+
+  signal_response().connect
+    (sigc::mem_fun(*this, &DocumentDialog::on_response_received));
+  xml->get_widget("title_entry", title_entry);
+  xml->get_widget("description_textview", description_textview);
+  xml->get_widget("thumbnail_image", thumbnail_image);
+  xml->get_widget("lat_spinbutton", lat_spinbutton);
+  xml->get_widget("lon_spinbutton", lon_spinbutton);
+  xml->get_widget("item_tagging_widget_container", 
+                  item_tagging_widget_container);
+  xml->get_widget("title_label", title_label);
+  xml->get_widget("description_label", description_label);
+  xml->get_widget("latitude_label", latitude_label);
+  xml->get_widget("longitude_label", longitude_label);
+
+
 }
 
 DocumentDialog::~DocumentDialog()
 {
-  delete dialog;
 }
 
-void DocumentDialog::set_parent_window(Gtk::Window &parent)
+void DocumentDialog::on_response_received(int response)
 {
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void DocumentDialog::hide()
-{
-  dialog->hide();
-}
-
-int DocumentDialog::run()
-{
-  int response = dialog->run();
   if (response == Gtk::RESPONSE_ACCEPT && document)
     {
-      document->set_lat_long(lat_spinbutton->get_value(), lon_spinbutton->get_value());
+      document->set_lat_long(lat_spinbutton->get_value(), 
+                             lon_spinbutton->get_value());
       document->set_title(title_entry->get_text());
       document->set_description(description_textview->get_buffer()->get_text());
       document->set_tags(item_tagging_widget->get_tags());
     }
-  return response;
 }
 
 void DocumentDialog::on_title_changed()
