@@ -24,25 +24,22 @@
 #include "item-tagging-widget.h"
 using namespace Tapioca;
 
-class DocumentDialog
+class DocumentDialog: public Gtk::Dialog
 {
  public:
-    DocumentDialog(Tapioca::Document *g, const Tapioca::Document *original,
-                   bool readonly = false);
-    ~DocumentDialog();
+    static DocumentDialog *create(Tapioca::Document *g, 
+                                  const Tapioca::Document *original,
+                                  bool readonly = false);
+    
+    DocumentDialog(BaseObjectType* baseObject, 
+                   const Glib::RefPtr<Gtk::Builder>& xml);
 
-    void set_parent_window(Gtk::Window &parent);
-
-    int run();
-    void hide();
-
- protected:
+    virtual ~DocumentDialog();
 
  private:
     Tapioca::Document *document;
     const Tapioca::Document *original;
     bool readonly;
-    Gtk::Dialog* dialog;
     Gtk::Entry *title_entry;
     Gtk::TextView *description_textview;
     Gtk::Image *thumbnail_image;
@@ -60,6 +57,10 @@ class DocumentDialog
     void on_description_changed();
     void on_lat_changed();
     void on_lon_changed();
+    void on_dialog_loaded(Tapioca::Document *g, 
+                          const Tapioca::Document *original, 
+                          bool readonly = false);
+    void on_response_received(int response);
 
 };
 
