@@ -1,4 +1,4 @@
-// Copyright (C) 2010 Ben Asselstine
+// Copyright (C) 2010, 2011 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@ void Document::set_thumbnail(std::string filename)
         }
       fclose(fileptr);
     }
+  thumbnail_filename = "";
 }
 
 void Document::update_thumbnail_from_loaded_data()
@@ -80,6 +81,7 @@ void Document::update_thumbnail_from_loaded_data()
   close(fd);
   Glib::file_set_contents(temp_filename, data, len);
   thumbnail = Gdk::Pixbuf::create_from_file(temp_filename);
+  thumbnail_filename = "";
   remove(temp_filename.c_str());
 }
 
@@ -207,6 +209,7 @@ bool Document::exists(std::string f)
     }
   return false;
 }
+
 void Document::clean_dir(std::string dirname)
 {
   if (Document::exists(dirname) == false)
@@ -250,3 +253,13 @@ std::string Document::get_export_filename() const
     }
   return get_id() + "-" + get_original_filename();
 }
+
+bool Document::same_except_for_tags(const Document& document) const
+{
+  Document another_document(document);
+  another_document.set_tags(get_tags());
+  if (another_document == *this)
+    return true;
+  return false;
+}
+
