@@ -1,4 +1,4 @@
-// Copyright (C) 2010 Ben Asselstine
+// Copyright (C) 2010, 2011 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -80,6 +80,7 @@ class Document: public Item
         static int remove_file(std::string filename);
         static Glib::ustring find_free_file(std::string file);
 
+        bool same_except_for_tags(const Document& document) const;
         //operators
         bool operator==(const Document &other)
           {
@@ -91,7 +92,6 @@ class Document: public Item
               gallery_id == other.gallery_id &&
               order == other.order &&
               tags == other.tags &&
-              thumbnail_filename == other.thumbnail_filename &&
               thumbnail_file_data == other.thumbnail_file_data &&
               image_filename == other.image_filename &&
               video_filename == other.video_filename &&
@@ -125,14 +125,11 @@ class Document: public Item
               ar & gallery_id;
               ar & order;
               ar & tags;
-              ar & thumbnail_filename;
               ar & thumbnail_file_data;
               ar & image_filename;
               ar & video_filename;
               ar & original_filename;
             }
-                
-
 };
 }
 
