@@ -22,19 +22,28 @@
 #include "main.h"
 #include "ucompose.hpp"
 
-GalleryDialog::GalleryDialog(Tapioca::Gallery *g,
-                             const Tapioca::Gallery *orig, bool uneditable)
+GalleryDialog* GalleryDialog::create(Tapioca::Gallery *g, const Tapioca::Gallery *orig, bool uneditable)
 {
-  gallery = g;
-  original = orig;
-  readonly = uneditable;
   Glib::RefPtr<Gtk::Builder> xml
     = Gtk::Builder::create_from_file(Main::get_glade_path()
                                      + "/gallery-dialog.gtk");
+  GalleryDialog *dialog;
+  xml->get_widget_derived("dialog", dialog);
+  dialog->on_dialog_loaded(g, orig, uneditable);
+  return dialog;
+}
 
-  xml->get_widget("dialog", dialog);
-  dialog->set_icon_from_file(Main::get_data_path() + "/icon.png");
+GalleryDialog::GalleryDialog(BaseObjectType* baseObject, 
+                                     const Glib::RefPtr<Gtk::Builder>& xml)
+ : Gtk::Dialog(baseObject)
+{
+  gallery = NULL;
+  original = NULL;
+  readonly = false;
+  set_icon_from_file(Main::get_data_path() + "/icon.png");
 
+  signal_response().connect
+    (sigc::mem_fun(*this, &GalleryDialog::on_response_received));
   xml->get_widget("title_entry", title_entry);
   xml->get_widget("description_textview", description_textview);
   xml->get_widget("thumbnail_image", thumbnail_image);
@@ -47,25 +56,27 @@ GalleryDialog::GalleryDialog(Tapioca::Gallery *g,
   xml->get_widget("description_label", description_label);
   xml->get_widget("latitude_label", latitude_label);
   xml->get_widget("longitude_label", longitude_label);
+  xml->get_widget("private_audience_checkbutton", private_audience_checkbutton);
 
+}
+
+void GalleryDialog::on_dialog_loaded(Tapioca::Gallery *g,
+                                     const Tapioca::Gallery *orig, 
+                                     bool uneditable)
+{
+  gallery = g;
+  original = orig;
+  readonly = uneditable;
   if (gallery)
     {
-      title_entry->set_text(g->get_title());
-      description_textview->get_buffer()->set_text(g->get_description());
-      lat_spinbutton->set_value(g->get_latitude());
-      lon_spinbutton->set_value(g->get_longitude());
-      thumbnail_image->set(g->get_thumbnail_image());
+      title_entry->set_text(gallery->get_title());
+      description_textview->get_buffer()->set_text(gallery->get_description());
+      lat_spinbutton->set_value(gallery->get_latitude());
+      lon_spinbutton->set_value(gallery->get_longitude());
+      thumbnail_image->set(gallery->get_thumbnail_image());
+      private_audience_checkbutton->set_active(gallery->get_private_audience());
     }
 
-  title_entry->signal_changed().connect
-    (sigc::mem_fun(this,&GalleryDialog::on_title_changed));
-  description_textview->get_buffer()->signal_changed().connect
-    (sigc::mem_fun(this, &GalleryDialog::on_description_changed));
-  lat_spinbutton->signal_value_changed().connect
-    (sigc::mem_fun(this,&GalleryDialog::on_lat_changed));
-  lon_spinbutton->signal_value_changed().connect
-    (sigc::mem_fun(this,&GalleryDialog::on_lon_changed));
-
   item_tagging_widget = new ItemTaggingWidget();
   item_tagging_widget->get_widget()->reparent(*item_tagging_widget_container);
   item_tagging_widget->fill_tags(gallery->get_tags());
@@ -83,27 +94,25 @@ GalleryDialog::GalleryDialog(Tapioca::Gallery *g,
   on_description_changed();
   on_lat_changed();
   on_lon_changed();
+  on_private_audience_toggled();
+  title_entry->signal_changed().connect
+    (sigc::mem_fun(this,&GalleryDialog::on_title_changed));
+  description_textview->get_buffer()->signal_changed().connect
+    (sigc::mem_fun(this, &GalleryDialog::on_description_changed));
+  lat_spinbutton->signal_value_changed().connect
+    (sigc::mem_fun(this,&GalleryDialog::on_lat_changed));
+  lon_spinbutton->signal_value_changed().connect
+    (sigc::mem_fun(this,&GalleryDialog::on_lon_changed));
+  private_audience_checkbutton->signal_toggled().connect
+    (sigc::mem_fun(this,&GalleryDialog::on_private_audience_toggled));
 }
 
 GalleryDialog::~GalleryDialog()
 {
-  delete dialog;
 }
 
-void GalleryDialog::set_parent_window(Gtk::Window &parent)
+void GalleryDialog::on_response_received(int response)
 {
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void GalleryDialog::hide()
-{
-  dialog->hide();
-}
-
-int GalleryDialog::run()
-{
-  int response = dialog->run();
   if (response == Gtk::RESPONSE_ACCEPT && gallery)
     {
       gallery->set_lat_long(lat_spinbutton->get_value(), lon_spinbutton->get_value());
@@ -111,7 +120,6 @@ int GalleryDialog::run()
       gallery->set_description(description_textview->get_buffer()->get_text());
       gallery->set_tags(item_tagging_widget->get_tags());
     }
-  return response;
 }
 
 void GalleryDialog::on_title_changed()
@@ -171,4 +179,21 @@ void GalleryDialog::on_lon_changed()
     }
   longitude_label->set_attributes(attrs);
 }
-
+    
+void GalleryDialog::on_private_audience_toggled()
+{
+  if (gallery)
+    {
+      gallery->set_private_audience(private_audience_checkbutton->get_active());
+    }
+  Gtk::Label *l = dynamic_cast<Gtk::Label*>(private_audience_checkbutton->get_child());
+  Pango::AttrList attrs;
+  if (original && gallery &&
+      original->get_private_audience() != gallery->get_private_audience())
+    {
+      Pango::Attribute weight = 
+        Pango::Attribute::create_attr_weight(Pango::WEIGHT_BOLD);
+      attrs.insert(weight);
+    }
+  l->set_attributes(attrs);
+}
