@@ -24,25 +24,18 @@
 #include "item-tagging-widget.h"
 using namespace Tapioca;
 
-class GalleryDialog
+class GalleryDialog: public Gtk::Dialog
 {
  public:
-    GalleryDialog(Tapioca::Gallery *g, const Tapioca::Gallery *original,
-                  bool readonly = false);
-    ~GalleryDialog();
+    virtual ~GalleryDialog();
 
-    void set_parent_window(Gtk::Window &parent);
-
-    int run();
-    void hide();
-
- protected:
+    GalleryDialog(BaseObjectType* baseObject, const Glib::RefPtr<Gtk::Builder>& xml);
 
+    static GalleryDialog* create(Tapioca::Gallery *g, const Tapioca::Gallery *original, bool readonly = false);
  private:
     Tapioca::Gallery *gallery;
     const Tapioca::Gallery *original;
     bool readonly;
-    Gtk::Dialog* dialog;
     Gtk::Entry *title_entry;
     Gtk::TextView *description_textview;
     Gtk::Image *thumbnail_image;
@@ -55,12 +48,17 @@ class GalleryDialog
     Gtk::Label *description_label;
     Gtk::Label *latitude_label;
     Gtk::Label *longitude_label;
+    Gtk::CheckButton *private_audience_checkbutton;
 
     //callbacks
     void on_title_changed();
     void on_description_changed();
     void on_lat_changed();
     void on_lon_changed();
+    void on_response_received(int response);
+    void on_private_audience_toggled();
+    void on_dialog_loaded(Tapioca::Gallery *g, const Tapioca::Gallery *orig, 
+                          bool uneditable);
 };
 
 #endif // TAPIOCA_GALLERY_DIALOG_H
