@@ -1,4 +1,4 @@
-// Copyright (C) 2010 Ben Asselstine
+// Copyright (C) 2010, 2011 Ben Asselstine
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,26 +18,33 @@
 #include <fstream>
 #include "gallery.h"
 #include "document.h"
+#include <string.h>
+#include <stdlib.h>
 
 using namespace Tapioca;
 
+sigc::signal<void> Gallery::loading;
+sigc::signal<void> Gallery::saving;
+
 Gallery::Gallery(const Gallery& g)
-: Item(g), document_thumbnail_id(g.document_thumbnail_id)
+: Item(g), document_thumbnail_id(g.document_thumbnail_id), 
+    private_audience(g.private_audience)
 {
-  for (Gallery::const_iterator it = g.begin(); it != g.end(); it++)
+  for (Gallery::const_iterator it = g.get_docs().begin(); 
+       it != g.get_docs().end(); it++)
     push_back(new Document(*(*it)));
 }
 
 Gallery::Gallery()
-: Item(), document_thumbnail_id("")
+: Item(), document_thumbnail_id(""), private_audience(false)
 {
 }
 
 void Gallery::remove_all_documents()
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _docs.begin(); it != _docs.end(); it++)
     delete (*it);
-  clear();
+  _docs.clear();
 }
 
 Gallery::~Gallery()
@@ -45,10 +52,9 @@ Gallery::~Gallery()
   remove_all_documents();
 }
 
-        
-Glib::RefPtr<Gdk::Pixbuf> Gallery::get_thumbnail_image() const
+const Glib::RefPtr<Gdk::Pixbuf> Gallery::get_thumbnail_image() const
 {
-  for (Gallery::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_docs().begin(); it != get_docs().end(); it++)
     {
       if ((*it)->get_id() == document_thumbnail_id)
         return (*it)->get_thumbnail();
@@ -77,10 +83,10 @@ std::string Gallery::get_next_in_order_sequence(std::string seq)
 
 void Gallery::set_document_order(Document *document, Document *after)
 {
-  std::string new_order = "";
-  if (after == NULL && front() != document && document)
+  std::string new_order;
+  if (after == NULL && _docs.front() != document && document)
     new_order = "";
-  else if (after == NULL && front() == document && document)
+  else if (after == NULL && _docs.front() == document && document)
     return;
   else
     {
@@ -99,7 +105,7 @@ void Gallery::set_document_order(Document *document, Document *after)
 
 bool Gallery::has_order(std::string o) const
 {
-  for (Gallery::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_docs().begin(); it != get_docs().end(); it++)
     {
       if ((*it)->get_order() == o)
         return true;
@@ -113,7 +119,7 @@ void Gallery::remove_document(Document *document)
   if (document_thumbnail_id == document->get_id())
     {
       if (size() > 0)
-        document_thumbnail_id = front()->get_id();
+        document_thumbnail_id = _docs.front()->get_id();
       else
         document_thumbnail_id = "";
     }
@@ -121,7 +127,7 @@ void Gallery::remove_document(Document *document)
 
 void Gallery::update_thumbnails()
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _docs.begin(); it != _docs.end(); it++)
     (*it)->update_thumbnail_from_loaded_data();
 }
 
@@ -130,12 +136,12 @@ void Gallery::add_document(Document *document)
   push_back(document);
   document->set_gallery_id(get_id());
   if (size() == 1)
-    document_thumbnail_id = front()->get_id();
+    document_thumbnail_id = _docs.front()->get_id();
 }
 
 void Gallery::find_deleted_documents(std::list<Document*> &removed)
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_docs().begin(); it != get_docs().end(); it++)
     {
       Document *document = (*it);
       if (document->get_image_filename() != "")
@@ -161,7 +167,7 @@ void Gallery::export_html_page(std::string export_dir) const
   ofs << "\t</body>\n</html>" << std::endl;
   ofs << "\t\t<div class=\"gallerytitleblock\">" << std::endl;
   ofs <<"\t\t\t" << Glib::Markup::escape_text(get_title()) << std::endl;
-  for (const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_docs().begin(); it != get_docs().end(); it++)
     {
       Document *doc = *it;
       if (doc->get_export_filename() == "")
@@ -206,12 +212,39 @@ void Gallery::export_html_page(std::string export_dir) const
 void Gallery::set_id(std::string i)
 {
   id = i;
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _docs.begin(); it != _docs.end(); it++)
     (*it)->set_gallery_id(i);
 }
         
 void Gallery::add(const Gallery*gallery)
 {
-  for (const_iterator i = gallery->begin(); i != gallery->end(); i++)
+  for (const_iterator i = gallery->get_docs().begin(); 
+       i != gallery->get_docs().end(); i++)
     push_back(new Document(*(*i)));
 }
+        
+bool Gallery::same_except_for_thumbnail_id(const Gallery& gallery) const
+{
+  Gallery another_gallery(gallery);
+  another_gallery.set_thumbnail(get_thumbnail_id());
+  if (another_gallery == *this)
+    return true;
+  return false;
+}
+
+bool Gallery::same_except_for_tags(const Gallery& gallery) const
+{
+  Gallery another_gallery(gallery);
+  another_gallery.set_tags(get_tags());
+  if (another_gallery == *this)
+    return true;
+  return false;
+}
+
+Document* Gallery::find_by_id(std::string i) const
+{
+  for (const_iterator j = get_docs().begin(); j != get_docs().end(); j++)
+    if ((*j)->get_id() == i)
+      return (*j);
+  return NULL;
+}
