@@ -25,13 +25,14 @@ using namespace Tapioca;
 
 GalleryList::GalleryList(const GalleryList& g)
 {
-  for (GalleryList::const_iterator it = g.begin(); it != g.end(); it++)
-    push_back(new Gallery(*(*it)));
+  for (const_iterator it = g.get_galleries().begin(); 
+       it != g.get_galleries().end(); it++)
+    _galleries.push_back(new Gallery(*(*it)));
 }
 
 Gallery* GalleryList::find_by_id(const std::string id) const
 {
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = _galleries.begin(); it != _galleries.end(); it++)
     {
       if ((*it)->get_id() == id)
         return (*it);
@@ -41,10 +42,11 @@ Gallery* GalleryList::find_by_id(const std::string id) const
 
 Document * GalleryList::find_doc_by_id(const std::string id) const
 {
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); 
+       it != get_galleries().end(); it++)
     {
-      for (Gallery::const_iterator git = (*it)->begin(); git != (*it)->end(); 
-           git++)
+      for (Gallery::const_iterator git = (*it)->get_docs().begin(); 
+           git != (*it)->get_docs().end(); git++)
         {
           if ((*git)->get_id() == id)
             return (*git);
@@ -57,8 +59,9 @@ void GalleryList::add(const GalleryList *g)
 {
   if (g)
     {
-      for (GalleryList::const_iterator it = g->begin(); it != g->end(); it++)
-        push_back(new Gallery(*(*it)));
+      for (const_iterator it = g->get_galleries().begin();
+           it != g->get_galleries().end(); it++)
+        _galleries.push_back(new Gallery(*(*it)));
     }
 }
 
@@ -66,7 +69,8 @@ std::list<ChangedGallery> GalleryList::get_changed_galleries(const GalleryList *
 {
   std::list<ChangedGallery> difflist;
   //which one of these is different from those?
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); 
+       it != get_galleries().end(); it++)
     {
       //does this id appear in the given list?
       Gallery *gallery = g->find_by_id((*it)->get_id());
@@ -76,8 +80,11 @@ std::list<ChangedGallery> GalleryList::get_changed_galleries(const GalleryList *
             ;
           else
             {
-              ChangedGallery changed_gallery(*(*it), *gallery);
-              difflist.push_back(changed_gallery);
+              if (gallery->same_except_for_thumbnail_id(*(*it)) == false)
+                {
+                  ChangedGallery changed_gallery(*(*it), *gallery);
+                  difflist.push_back(changed_gallery);
+                }
             }
         }
     }
@@ -87,7 +94,8 @@ std::list<ChangedGallery> GalleryList::get_changed_galleries(const GalleryList *
 std::list<ChangedGallery> GalleryList::get_removed_galleries(const GalleryList *g) const
 {
   std::list<ChangedGallery> difflist;
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); 
+       it != get_galleries().end(); it++)
     {
       Gallery *gallery = g->find_by_id((*it)->get_id());
       if (gallery == NULL)
@@ -99,32 +107,14 @@ std::list<ChangedGallery> GalleryList::get_removed_galleries(const GalleryList *
   return difflist;
 }
 
-
-void GalleryList::show(Gallery *gallery)
-{
-  printf("showing gallery:\n");
-  printf("id: %s\n", gallery->get_id().c_str());
-  printf("lat: %lf\t long: %lf\n", gallery->get_latitude(), gallery->get_longitude());
-  printf("title: %s\n", gallery->get_title().c_str());
-  printf("description: %s\n", gallery->get_description().c_str());
-  printf("doc thumb id: %s\n", gallery->get_thumbnail_id().c_str());
-  printf("order: %s\n", gallery->get_order().c_str());
-  printf("tags:\n");
-  std::list<std::string> tags = gallery->get_tags();
-  for (std::list<std::string>::iterator it = tags.begin(); it != tags.end();
-       it++)
-    {
-      printf("\t'%s'\n", (*it).c_str());
-    }
-}
-
 std::list<ChangedDocument> GalleryList::get_changed_documents(const GalleryList *g) const
 {
   std::list<ChangedDocument> difflist;
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); 
+       it != get_galleries().end(); it++)
     {
-      for (Gallery::const_iterator git = (*it)->begin(); git != (*it)->end(); 
-           git++)
+      for (Gallery::const_iterator git = (*it)->get_docs().begin(); 
+           git != (*it)->get_docs().end(); git++)
         {
           Document *document = g->find_doc_by_id((*git)->get_id());
           if (document != NULL)
@@ -132,7 +122,10 @@ std::list<ChangedDocument> GalleryList::get_changed_documents(const GalleryList
               if (*document == *(*git))
                 ;
               else
-                difflist.push_back(ChangedDocument(*(*git), *document));
+                {
+                  if (document->get_gallery_id() == (*git)->get_gallery_id())
+                    difflist.push_back(ChangedDocument(*(*git), *document));
+                }
             }
         }
     }
@@ -142,14 +135,19 @@ std::list<ChangedDocument> GalleryList::get_changed_documents(const GalleryList
 std::list<ChangedDocument> GalleryList::get_removed_documents(const GalleryList *g) const
 {
   std::list<ChangedDocument> difflist;
-  for (GalleryList::const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); it != get_galleries().end(); it++)
     {
-      for (Gallery::const_iterator git = (*it)->begin(); git != (*it)->end(); 
-           git++)
+      for (Gallery::const_iterator git = (*it)->get_docs().begin(); 
+           git != (*it)->get_docs().end(); git++)
         {
           Document *document = g->find_doc_by_id((*git)->get_id());
           if (document == NULL)
             difflist.push_back(ChangedDocument(*(*git), Document()));
+          else
+            {
+              if (document->get_gallery_id() != (*git)->get_gallery_id())
+                difflist.push_back(ChangedDocument(*(*git), Document()));
+            }
         }
     }
   return difflist;
@@ -166,27 +164,28 @@ GalleryList::~GalleryList()
 
 void GalleryList::update_thumbnails()
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _galleries.begin(); it != _galleries.end(); it++)
     (*it)->update_thumbnails();
 }
 
 void GalleryList::find_deleted_documents(std::list<Document*> &removed_docs)
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _galleries.begin(); it != _galleries.end(); it++)
     (*it)->find_deleted_documents(removed_docs);
 }
 
 void GalleryList::remove_all_galleries()
 {
-  for (iterator it = begin(); it != end(); it++)
+  for (iterator it = _galleries.begin(); it != _galleries.end(); it++)
     delete (*it);
-  clear();
+  _galleries.clear();
 }
 
 size_t GalleryList::count_documents() const
 {
   size_t count = 0;
-  for (const_iterator it = begin(); it != end(); it++)
+  for (const_iterator it = get_galleries().begin(); 
+       it != get_galleries().end(); it++)
     count += (*it)->size();
   return count;
 }
@@ -202,7 +201,8 @@ void GalleryList::export_html_pages(std::string u, std::string export_dir) const
   ofs << "\t\t<div class=\"maintitleblock\">" << std::endl;
   ofs <<"\t\t" << Glib::Markup::escape_text(u) << "'s Galleries" << std::endl;
   ofs << "\t\t</div>" << std::endl;
-  for (const_iterator i = begin(); i != end(); i++)
+  for (const_iterator i = get_galleries().begin(); 
+       i != get_galleries().end(); i++)
     {
       Gallery *g = *i;
       ofs << "\t\t<div class=\"galleryblock\">" << std::endl;
@@ -247,6 +247,38 @@ void GalleryList::export_html_pages(std::string u, std::string export_dir) const
     }
   ofs << "\t</body>\n</html>" << std::endl;
   ofs.close();
-  for (const_iterator j = begin(); j != end(); j++)
+  for (const_iterator j = get_galleries().begin(); 
+       j != get_galleries().end(); j++)
     (*j)->export_html_page(export_dir);
 }
+
+bool GalleryList::is_same(const GalleryList &g) const
+{
+  if (size() != g.size())
+    return false;
+  for (const_iterator i = get_galleries().begin();
+       i != get_galleries().end(); i++)
+    {
+      Gallery *gallery = g.find_by_id((*i)->get_id());
+      if (!gallery)
+        return false;
+      if (gallery->size() != (*i)->size())
+        return false;
+      if (*gallery == *(*i))
+        ;
+      else
+        return false;
+      for (Gallery::const_iterator j = (*i)->get_docs().begin(); 
+           j != (*i)->get_docs().end(); j++)
+        {
+          Document *document = gallery->find_by_id((*j)->get_id());
+          if (document == NULL)
+            return false;
+          if (*document == *(*j))
+            ;
+          else
+            return false;
+        }
+    }
+  return true;
+}
