@@ -25,13 +25,14 @@
 #include <boost/archive/text_oarchive.hpp>
 #include <boost/archive/text_iarchive.hpp>
 #include <boost/serialization/list.hpp>
+#include "gallery.h"
 namespace Tapioca
 {
 class Gallery;
 class Document;
 class ChangedGallery;
 class ChangedDocument;
-class GalleryList: public std::list<Gallery*>
+class GalleryList
 {
     public:
 
@@ -43,7 +44,9 @@ class GalleryList: public std::list<Gallery*>
 
 	//! Destructor.
         virtual ~GalleryList();
-
+        
+        typedef std::list<Gallery*>::iterator iterator;
+        typedef std::list<Gallery*>::const_iterator const_iterator;
 
         void add(const GalleryList *g);
         std::list<ChangedGallery> get_changed_galleries(const GalleryList *g) const;
@@ -53,24 +56,31 @@ class GalleryList: public std::list<Gallery*>
         Gallery* find_by_id(std::string id) const;
         Document* find_doc_by_id(const std::string id) const;
         
+        size_t size() const {return _galleries.size();}
+        Gallery* back() {return _galleries.back();}
+        void remove(Gallery *gallery) {_galleries.remove(gallery);}
+        void push_back(Gallery *gallery) {_galleries.push_back(gallery);}
+
+        bool is_same(const GalleryList &g) const;
         void update_thumbnails();
         void find_deleted_documents(std::list<Document*> &removed);
         void remove_all_galleries();
         size_t count_documents() const;
         void export_html_pages(std::string username, std::string dir) const;
-    protected:
 
+        //std::list<Gallery*>& get_galleries() {return _galleries;}
+        const std::list<Gallery*>& get_galleries()const {return _galleries;}
     private:
 
-        static void show(Tapioca::Gallery *gallery);
-
         friend class boost::serialization::access;
                              
         template<class Archive>
           void serialize(Archive & ar, const unsigned int version)
             {
-              ar & static_cast<std::list<Gallery*>&>(*this);
+              //ar & static_cast<std::list<Gallery*>&>(*this);
+              ar & _galleries;
             }
+        std::list<Gallery*> _galleries;
 
 };
 }
