@@ -33,3 +33,12 @@ Item::Item()
 Item::~Item()
 {
 }
+
+std::string Item::get_tags_as_string() const
+{
+  std::string t;
+  for (std::list<std::string>::const_iterator i = tags.begin(); 
+       i != tags.end(); i++)
+    t += (*i) + " ";
+  return t;
+}
