@@ -55,19 +55,20 @@ class Item
 
         std::string get_id() const {return id;}
         void get_lat_long(double &lat, double &lon) const {lat=latitude;lon=longitude;}
-        double get_latitude() const {return latitude;}
-        double get_longitude () const {return longitude;}
+        const double& get_latitude() const {return latitude;}
+        const double& get_longitude () const {return longitude;}
 
-        std::string get_title() const {return title;}
+        const std::string& get_title() const {return title;}
 
-        std::string get_description() const {return description;}
+        const std::string& get_description() const {return description;}
 
-        std::string get_order() const {return order;}
+        const std::string& get_order() const {return order;}
 
         std::list<std::string> get_tags() const {return tags;}
 
         bool has_tag(std::string tag) const;
 
+        std::string get_tags_as_string() const;
     protected:
 
         std::string id;
