@@ -21,14 +21,11 @@
 #include "preferences-dialog.h"
 #include "main.h"
 
-PreferencesDialog::PreferencesDialog()
+PreferencesDialog::PreferencesDialog(BaseObjectType* baseObject, 
+                                     const Glib::RefPtr<Gtk::Builder>& xml)
+ : Gtk::Dialog(baseObject)
 {
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(Main::get_glade_path()
-				    + "/preferences-dialog.gtk");
-
-  xml->get_widget("dialog", dialog);
-  dialog->set_icon_from_file(Main::get_data_path() + "/icon.png");
+  set_icon_from_file(Main::get_data_path() + "/icon.png");
 
   xml->get_widget("homepage_entry", homepage_entry);
   homepage_entry->signal_key_release_event().connect
@@ -74,37 +71,32 @@ PreferencesDialog::PreferencesDialog()
 
   conf_client->notify_add("/apps/" PACKAGE_NAME "/general/video_viewer_path",
                           sigc::mem_fun(*this, 
-                                        &PreferencesDialog::key_changed));
+                                        &PreferencesDialog::on_key_changed));
   conf_client->notify_add("/apps/" PACKAGE_NAME "/general/image_viewer_path",
                             sigc::mem_fun(*this, 
-                                          &PreferencesDialog::key_changed));
+                                          &PreferencesDialog::on_key_changed));
   conf_client->notify_add("/apps/" PACKAGE_NAME "/general/homepage",
                           sigc::mem_fun(*this, 
-                                        &PreferencesDialog::key_changed));
+                                        &PreferencesDialog::on_key_changed));
+}
+
+PreferencesDialog* PreferencesDialog::create()
+{
+  Glib::RefPtr<Gtk::Builder> xml
+    = Gtk::Builder::create_from_file(Main::get_glade_path()
+				    + "/preferences-dialog.gtk");
+
+  PreferencesDialog *dialog;
+  xml->get_widget_derived("dialog", dialog);
+  return dialog;
 }
 
 PreferencesDialog::~PreferencesDialog()
 {
-  delete dialog;
-}
-void PreferencesDialog::set_parent_window(Gtk::Window &parent)
-{
-    dialog->set_transient_for(parent);
-    //dialog->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
-}
-
-void PreferencesDialog::hide()
-{
-  dialog->hide();
-}
-
-void PreferencesDialog::run()
-{
-  dialog->run();
 }
 
 void
-PreferencesDialog::key_changed(guint connection_id, Gnome::Conf::Entry entry)
+PreferencesDialog::on_key_changed(guint connection_id, Gnome::Conf::Entry entry)
 {
   // Make sure the preference has a valid value
   if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING) 
@@ -148,7 +140,7 @@ bool PreferencesDialog::on_homepage_changed(GdkEventKey *event)
 
 void PreferencesDialog::on_video_viewer_clicked()
 {
-  Gtk::FileChooserDialog chooser(*dialog, "Choose Program To View Videos");
+  Gtk::FileChooserDialog chooser(*this, "Choose Program To View Videos");
   chooser.set_current_folder("/usr/bin");
 
   chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
@@ -168,7 +160,7 @@ void PreferencesDialog::on_video_viewer_clicked()
 
 void PreferencesDialog::on_image_viewer_clicked()
 {
-  Gtk::FileChooserDialog chooser(*dialog, "Choose Program To View Images");
+  Gtk::FileChooserDialog chooser(*this, "Choose Program To View Images");
   chooser.set_current_folder("/usr/bin");
 
   chooser.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
