@@ -22,22 +22,18 @@
 #include <gconfmm.h>
 
 
-class PreferencesDialog
+class PreferencesDialog: public Gtk::Dialog
 {
  public:
-    PreferencesDialog();
-    ~PreferencesDialog();
 
-    void set_parent_window(Gtk::Window &parent);
+     PreferencesDialog(BaseObjectType* baseObject, 
+                       const Glib::RefPtr<Gtk::Builder>& xml);
+    
+     virtual ~PreferencesDialog();
 
-    void run();
-    void hide();
-
- protected:
-    void key_changed(guint connection_id, Gnome::Conf::Entry entry);
+     static PreferencesDialog* create();
 
  private:
-    Gtk::Dialog* dialog;
     Gtk::Entry *homepage_entry;
     Gtk::Button *video_viewer_button;
     Gtk::Button *image_viewer_button;
@@ -50,6 +46,7 @@ class PreferencesDialog
     bool on_homepage_changed(GdkEventKey *event);
     void on_video_viewer_clicked();
     void on_image_viewer_clicked();
+    void on_key_changed(guint connection_id, Gnome::Conf::Entry entry);
 
     //helpers
     void update_viewer_button(Gtk::Button *button, Glib::ustring file);
