@@ -21,20 +21,20 @@
 #include <gtkmm.h>
 #include "session.h"
 
-class PublishDialog
+class PublishDialog: public Gtk::Dialog
 {
  public:
-    PublishDialog(const Tapioca::SessionChanges &changes);
-    ~PublishDialog();
-
-    void set_parent_window(Gtk::Window &parent);
-
-    int run();
-    void hide();
+    static PublishDialog* create(const Tapioca::SessionChanges &changes);
+    PublishDialog(BaseObjectType* baseObject, 
+                  const Glib::RefPtr<Gtk::Builder>& xml);
+    
+    virtual ~PublishDialog();
 
  private:
-    Gtk::Dialog* dialog;
     Gtk::Label *changes_label;
+
+    //callbacks
+    void on_dialog_loaded(const Tapioca::SessionChanges &changes);
 };
 
 #endif // TAPIOCA_PUBLISH_DIALOG_H
