@@ -27,17 +27,16 @@
 #define TAPIOCA_EXT ".tao"
     
 class ProgressDialog;
-class TapiocaWindow
+class TapiocaWindow: public Gtk::Window
 {
  public:
-    TapiocaWindow(std::string file = "");
+    static TapiocaWindow* create(std::string file = "");
 
-    Gtk::Window *get_window() {return window.get();};
-    bool run();
-    void hide();
-    
+    TapiocaWindow(BaseObjectType* baseObject, 
+                  const Glib::RefPtr<Gtk::Builder>& xml);
+
+    virtual ~TapiocaWindow();
  private:
-    std::auto_ptr<Gtk::Window> window;
 
     //for the treeview
     class GalleriesColumns: public Gtk::TreeModelColumnRecord {
@@ -83,6 +82,7 @@ class TapiocaWindow
     Gtk::Menu iconview_context_menu;
 
     //callbacks
+    void on_window_loaded(std::string filename);
     void on_connect_clicked();
     void on_open_offline_changes_clicked();
     void on_save_offline_changes_clicked();
@@ -146,11 +146,41 @@ class TapiocaWindow
     void remove_deleted_documents();
     void update_window_title();
     void load_session_in_the_background(ProgressDialog &d);
+    void save_session_in_the_background(ProgressDialog &d);
     void export_documents_in_the_background(ProgressDialog &d, std::string export_dir, std::list<Tapioca::Document*> docs);
     void view_document_in_the_background(ProgressDialog &d, Tapioca::Document *document, const Tapioca::Session::SlotDownloadDocument slot);
     void pull_galleries_in_the_background(Tapioca::Session *s, ProgressDialog &d);
 
+    void on_published_gallery_added(Tapioca::Gallery *gallery, sigc::slot<void> slot);
+    void on_published_gallery_removed(Tapioca::Gallery *gallery, sigc::slot<void> slot);
+    void on_published_document_added(Tapioca::Document *document, sigc::slot<void> slot);
+    void on_published_document_removed(Tapioca::Document *document, sigc::slot<void> slot);
+    void on_published_gallery_modified(Tapioca::Gallery *gallery, sigc::slot<void> slot);
+    void on_published_document_modified(Tapioca::Document *document, sigc::slot<void> slot);
+    void on_published_galleries_added();
+    void on_published_galleries_removed();
+    void on_published_documents_added();
+    void on_published_documents_removed();
+    void on_published_galleries_modified();
+    void on_published_documents_modified();
+
+
+
+    void add_published_galleries(sigc::slot<void> slot);
+    void remove_published_galleries(sigc::slot<void> slot);
+    void add_published_documents(sigc::slot<void> slot);
+    void remove_published_documents(sigc::slot<void> slot);
+    void modify_published_galleries(sigc::slot<void> slot);
+    void modify_published_documents(sigc::slot<void> slot);
+
+    void move_published_documents(sigc::slot<void> slot);
+    void on_published_document_moved(Tapioca::Document *src, Tapioca::Document *dest, sigc::slot<void> slot);
+    void on_published_documents_moved();
+    void on_tags_updated(Tapioca::Item *item);
     std::string add_ext_if_necessary(std::string file, std::string ext) const;
+    void publish(Tapioca::SessionChanges &changes);
+    bool on_delete_event(GdkEventAny *event);
+    bool quit();
 
     //DATA
     std::string current_save_filename;
@@ -158,6 +188,8 @@ class TapiocaWindow
     bool needs_saving;
     std::string temporary_directory;
     bool progress_aborted;
+    Tapioca::SessionChanges publish_changes;
+    std::list<std::string> d_thumbnail_urls;
 };
 
 #endif // TAPIOCA_WINDOW_H
